from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.wait_instance_by_uuid_request_body_state import (
    WaitInstanceByUUIDRequestBodyState,
    check_wait_instance_by_uuid_request_body_state,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="WaitInstanceByUUIDRequestBody")


@_attrs_define
class WaitInstanceByUUIDRequestBody:
    """Wait parameters."""

    state: Union[Unset, WaitInstanceByUUIDRequestBodyState] = UNSET
    """ The desired state to wait for.  Default is `running`. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if state is not UNSET:
            field_dict["state"] = state

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _state = d.pop("state", UNSET)
        state: Union[Unset, WaitInstanceByUUIDRequestBodyState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = check_wait_instance_by_uuid_request_body_state(_state)

        wait_instance_by_uuid_request_body = cls(
            state=state,
        )

        wait_instance_by_uuid_request_body.additional_properties = d
        return wait_instance_by_uuid_request_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
